@@ -287,6 +287,7 @@ gnc_plugin_page_merge_actions (GncPluginPage *page,
 
     priv = GNC_PLUGIN_PAGE_GET_PRIVATE(page);
     priv->ui_merge = ui_merge;
+    gtk_action_group_set_sensitive (priv->action_group, TRUE);
     priv->merge_id = gnc_plugin_add_actions(priv->ui_merge,
                                             priv->action_group,
                                             priv->ui_description);
@@ -307,6 +308,7 @@ gnc_plugin_page_unmerge_actions (GncPluginPage *page,
     g_return_if_fail (priv->action_group != NULL);
 
     gtk_ui_manager_remove_ui(ui_merge, priv->merge_id);
+    gtk_action_group_set_sensitive (priv->action_group, FALSE);
     gtk_ui_manager_remove_action_group(ui_merge, priv->action_group);
 
     priv->ui_merge = NULL;
