@@ -60,13 +60,13 @@
 // static QofLogModule log_module = GNC_MOD_SX;
 static QofLogModule log_module = GNC_MOD_GUI;
 
-#define STATE_SECTION_REG_PREFIX "Register"
-
 /***** PROTOTYPES ***************************************************/
 void gnc_split_reg_raise( GNCSplitReg *gsr );
 
-static GtkWidget* add_summary_label( GtkWidget *summarybar,
-                                     const char *label_str );
+static GtkWidget* add_summary_label( GtkWidget *summarybar, gboolean pack_start,
+                                     const char *label_str, GtkWidget *extra );
+
+static void gsr_summarybar_set_arrow_draw (GNCSplitReg *gsr);
 
 static void gnc_split_reg_determine_read_only( GNCSplitReg *gsr );
 
@@ -159,7 +159,6 @@ void gnc_split_reg_sort_action_cb (GtkWidget *w, gpointer data);
 void gnc_split_reg_sort_notes_cb (GtkWidget *w, gpointer data);
 
 
-void gnc_split_reg_destroy_cb(GtkWidget *widget, gpointer data);
 void gnc_split_reg_size_allocate( GtkWidget *widget,
                                   GtkAllocation *allocation,
                                   gpointer user_data );
@@ -168,6 +167,7 @@ void gnc_split_reg_size_allocate( GtkWidget *widget,
 static void gnc_split_reg_class_init( GNCSplitRegClass *klass );
 static void gnc_split_reg_init( GNCSplitReg *gsr );
 static void gnc_split_reg_init2( GNCSplitReg *gsr );
+void gnc_split_reg_dispose(GObject *obj);
 
 FROM_STRING_FUNC(SortType, ENUM_LIST_SORTTYPE)
 AS_STRING_FUNC(SortType, ENUM_LIST_SORTTYPE)
@@ -314,6 +314,8 @@ gnc_split_reg_class_init( GNCSplitRegClass *klass )
     klass->help_changed_cb = NULL;
     klass->show_popup_menu_cb = NULL;
     klass->include_date_cb = NULL;
+
+    object_class->dispose = gnc_split_reg_dispose;
 }
 
 GtkWidget*
@@ -347,13 +349,13 @@ gnc_split_reg_init( GNCSplitReg *gsr )
     gtk_orientable_set_orientation (GTK_ORIENTABLE(gsr), GTK_ORIENTATION_VERTICAL);
 
     gsr->sort_type = BY_STANDARD;
+    gsr->sort_rev = FALSE;
+    gsr->sort_arrow_handler_id = 0;
+    gsr->filter_text = NULL;
     gsr->width = -1;
     gsr->height = -1;
     gsr->numRows = 10;
     gsr->read_only = FALSE;
-
-    g_signal_connect( gsr, "destroy",
-                      G_CALLBACK (gnc_split_reg_destroy_cb), gsr );
 }
 
 static void
@@ -441,8 +443,20 @@ gsr_setup_status_widgets( GNCSplitReg *gsr )
 }
 
 void
-gnc_split_reg_destroy_cb(GtkWidget *widget, gpointer data)
+gnc_split_reg_dispose(GObject *obj)
 {
+    GNCSplitReg *gsr = GNC_SPLIT_REG(obj);
+
+    if (gsr->filter_text)
+        g_free (gsr->filter_text);
+    gsr->filter_text = NULL;
+
+    if (gsr->reg)
+    {
+        g_signal_handlers_disconnect_by_data (gsr->reg, gsr);
+        gtk_widget_destroy (GTK_WIDGET (gsr->reg));
+    }
+    gsr->reg = NULL;
 }
 
 /**
@@ -531,21 +545,109 @@ gsr_redraw_all_cb (GnucashRegister *g_reg, gpointer data)
     print_info = gnc_account_print_info( leader, TRUE );
     reverse = gnc_reverse_balance( leader );
 
-    gsr_update_summary_label( gsr->balance_label,
-                              xaccAccountGetPresentBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->cleared_label,
-                              xaccAccountGetClearedBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->reconciled_label,
-                              xaccAccountGetReconciledBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->future_label,
-                              xaccAccountGetBalance,
-                              leader, print_info, commodity, reverse, euro );
-    gsr_update_summary_label( gsr->projectedminimum_label,
-                              xaccAccountGetProjectedMinimumBalance,
-                              leader, print_info, commodity, reverse, euro );
+    if (gsr->balance_label != NULL) // only test the first as they are a group
+    {
+        gsr_update_summary_label( gsr->balance_label,
+                                  xaccAccountGetPresentBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->cleared_label,
+                                  xaccAccountGetClearedBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->reconciled_label,
+                                  xaccAccountGetReconciledBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->future_label,
+                                  xaccAccountGetBalance,
+                                  leader, print_info, commodity, reverse, euro );
+        gsr_update_summary_label( gsr->projectedminimum_label,
+                                  xaccAccountGetProjectedMinimumBalance,
+                                  leader, print_info, commodity, reverse, euro );
+    }
+
+    // Sort label
+    if (gsr->sort_label != NULL)
+    {
+        gchar *old_tt_text = gtk_widget_get_tooltip_text (GTK_WIDGET(gsr->sort_label));
+        gchar *new_tt_text;
+        gchar *text = NULL;
+
+        switch (gsr->sort_type)
+        {
+        case (0):
+            text =  _("None");
+            break;
+        case (1):
+            text = _("Standard Order");
+            break;
+        case (2):
+            text = _("Date");
+            break;
+        case (3):
+            text = _("Date of Entry");
+            break;
+        case (4):
+            text = _("Statement Date");
+            break;
+        case (5):
+            text = _("Number");
+            break;
+        case (6):
+            text = _("Amount");
+            break;
+        case (7):
+            text = _("Memo");
+            break;
+        case (8):
+            text = _("Description");
+            break;
+        case (9):
+            text = _("Action");
+            break;
+        case (10):
+            text = _("Notes");
+            break;
+        }
+
+        if (gsr->sort_rev)
+            gtk_widget_set_tooltip_text (GTK_WIDGET(gsr->sort_label), _("Descending"));
+        else
+            gtk_widget_set_tooltip_text (GTK_WIDGET(gsr->sort_label), _("Ascending"));
+
+        new_tt_text = gtk_widget_get_tooltip_text (GTK_WIDGET(gsr->sort_label));
+
+        // does the arrow need changing
+        if (g_strcmp0 (old_tt_text, new_tt_text) != 0)
+            gsr_summarybar_set_arrow_draw (gsr);
+
+        if (old_tt_text)
+            g_free (old_tt_text);
+
+        if (new_tt_text)
+            g_free (new_tt_text);
+
+        gtk_label_set_text (GTK_LABEL(gsr->sort_label), text);
+    }
+
+    // Filter label
+    if (gsr->filter_label != NULL)
+    {
+        gchar *old_tt_text = gtk_widget_get_tooltip_text (GTK_WIDGET(gsr->filter_label));
+
+        // check for a change in text
+        if (g_strcmp0 (old_tt_text, gsr->filter_text) != 0)
+        {
+            if (gsr->filter_text != NULL)
+                gtk_label_set_text (GTK_LABEL(gsr->filter_label), _("Filtered"));
+            else
+                gtk_label_set_text (GTK_LABEL(gsr->filter_label), "");
+
+            gtk_widget_set_tooltip_text (GTK_WIDGET(gsr->filter_label), gsr->filter_text);
+
+            if (old_tt_text)
+                g_free (old_tt_text);
+        }
+    }
+
     if (gsr->shares_label == NULL && gsr->value_label == NULL)
         return;
     amount = xaccAccountGetBalance( leader );
@@ -615,7 +717,9 @@ gnc_split_reg_ld_destroy( GNCLedgerDisplay *ledger )
     }
     g_free (state_section);
     g_free (acct_fullname);
+
     gnc_ledger_display_set_user_data (ledger, NULL);
+    g_object_unref (gsr);
 }
 
 void
@@ -801,7 +905,7 @@ gnc_split_reg_reverse_trans_cb (GtkWidget *w, gpointer data)
 
 
 static gboolean
-is_trans_readonly_and_warn (GtkWindow *parent, const Transaction *trans)
+is_trans_readonly_and_warn (GtkWindow *parent, Transaction *trans)
 {
     GtkWidget *dialog;
     const gchar *reason;
@@ -1375,6 +1479,7 @@ gsr_default_schedule_handler( GNCSplitReg *gsr, gpointer data )
         ((guid_equal (xaccSchedXactionGetGUID (sx), fromSXId))
          ? sx : NULL);
     }
+    guid_free (fromSXId);
 
     if ( theSX )
     {
@@ -1654,7 +1759,7 @@ gnc_split_reg_jump_cb( GtkWidget *widget, gpointer data )
 }
 
 void
-gnc_split_reg_change_style (GNCSplitReg *gsr, SplitRegisterStyle style)
+gnc_split_reg_change_style (GNCSplitReg *gsr, SplitRegisterStyle style, gboolean refresh)
 {
     SplitRegister *reg = gnc_ledger_display_get_split_register (gsr->ledger);
 
@@ -1662,7 +1767,8 @@ gnc_split_reg_change_style (GNCSplitReg *gsr, SplitRegisterStyle style)
         return;
 
     gnc_split_register_config (reg, reg->type, style, reg->use_double_line);
-    gnc_ledger_display_refresh (gsr->ledger);
+    if (refresh)
+        gnc_ledger_display_refresh (gsr->ledger);
 }
 
 void
@@ -1673,7 +1779,7 @@ gnc_split_reg_style_ledger_cb (GtkWidget *w, gpointer data)
     if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w)))
         return;
 
-    gnc_split_reg_change_style (gsr, REG_STYLE_LEDGER);
+    gnc_split_reg_change_style (gsr, REG_STYLE_LEDGER, TRUE);
 }
 
 void
@@ -1684,7 +1790,7 @@ gnc_split_reg_style_auto_ledger_cb (GtkWidget *w, gpointer data)
     if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w)))
         return;
 
-    gnc_split_reg_change_style (gsr, REG_STYLE_AUTO_LEDGER);
+    gnc_split_reg_change_style (gsr, REG_STYLE_AUTO_LEDGER, TRUE);
 }
 
 void
@@ -1695,7 +1801,7 @@ gnc_split_reg_style_journal_cb (GtkWidget *w, gpointer data)
     if (!gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM(w)))
         return;
 
-    gnc_split_reg_change_style (gsr, REG_STYLE_JOURNAL);
+    gnc_split_reg_change_style (gsr, REG_STYLE_JOURNAL, TRUE);
 }
 
 void
@@ -1866,16 +1972,18 @@ gnc_split_reg_sort_notes_cb(GtkWidget *w, gpointer data)
 
 
 void
-gnc_split_reg_set_sort_reversed(GNCSplitReg *gsr, gboolean rev)
+gnc_split_reg_set_sort_reversed(GNCSplitReg *gsr, gboolean rev, gboolean refresh)
 {
-  /* Note: sort_reversed is the boolean opposite of sort_increasing
-   *       so when rev == true, we're sorting decreasing
-   *       In other words, qof_query_set_sort_increasing should
-   *       always use the inverse of rev.
-   */
-  Query *query = gnc_ledger_display_get_query( gsr->ledger );
-  qof_query_set_sort_increasing (query, !rev, !rev, !rev);
-  gnc_ledger_display_refresh( gsr->ledger );
+    /* Note: sort_reversed is the boolean opposite of sort_increasing
+     *       so when rev == true, we're sorting decreasing
+     *       In other words, qof_query_set_sort_increasing should
+     *       always use the inverse of rev.
+     */
+    Query *query = gnc_ledger_display_get_query( gsr->ledger );
+    qof_query_set_sort_increasing (query, !rev, !rev, !rev);
+    gsr->sort_rev = rev;
+    if (refresh)
+        gnc_ledger_display_refresh( gsr->ledger );
 }
 
 static void
@@ -2003,14 +2111,17 @@ gnc_split_reg_size_allocate (GtkWidget *widget,
 
 static
 GtkWidget*
-add_summary_label (GtkWidget *summarybar, const char *label_str)
+add_summary_label (GtkWidget *summarybar, gboolean pack_start, const char *label_str, GtkWidget *extra)
 {
     GtkWidget *hbox;
     GtkWidget *label;
 
     hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
     gtk_box_set_homogeneous (GTK_BOX (hbox), FALSE);
-    gtk_box_pack_start( GTK_BOX(summarybar), hbox, FALSE, FALSE, 5 );
+    if (pack_start)
+        gtk_box_pack_start( GTK_BOX(summarybar), hbox, FALSE, FALSE, 5 );
+    else
+        gtk_box_pack_end( GTK_BOX(summarybar), hbox, FALSE, FALSE, 5 );
 
     label = gtk_label_new( label_str );
     gnc_label_set_alignment(label, 1.0, 0.5 );
@@ -2020,44 +2131,65 @@ add_summary_label (GtkWidget *summarybar, const char *label_str)
     gnc_label_set_alignment(label, 1.0, 0.5 );
     gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 0 );
 
+    if (extra != NULL)
+        gtk_box_pack_start( GTK_BOX(hbox), extra, FALSE, FALSE, 0 );
+
     return label;
 }
 
+static void
+gsr_summarybar_set_arrow_draw (GNCSplitReg *gsr)
+{
+    if (gsr->sort_arrow_handler_id > 0)
+       g_signal_handler_disconnect (G_OBJECT(gsr->sort_arrow), gsr->sort_arrow_handler_id);
+
+    gsr->sort_arrow_handler_id = g_signal_connect (G_OBJECT (gsr->sort_arrow), "draw",
+                                                   G_CALLBACK (gnc_draw_arrow_cb), GINT_TO_POINTER(gsr->sort_rev));
+
+    gtk_widget_queue_draw (gsr->sort_arrow);
+}
+
 GtkWidget *
 gsr_create_summary_bar( GNCSplitReg *gsr )
 {
-    GtkWidget *summarybar;
+    GtkWidget *summarybar = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
+    gtk_box_set_homogeneous (GTK_BOX (summarybar), FALSE);
+    gtk_widget_set_name (summarybar, "gnc-id-summarybar");
 
     gsr->cleared_label    = NULL;
     gsr->balance_label    = NULL;
     gsr->reconciled_label = NULL;
     gsr->future_label     = NULL;
     gsr->projectedminimum_label  = NULL;
+    gsr->sort_label       = NULL;
+    gsr->sort_arrow       = NULL;
+    gsr->filter_label     = NULL;
     gsr->shares_label     = NULL;
     gsr->value_label      = NULL;
 
-    if ( gnc_ledger_display_type(gsr->ledger) >= LD_SUBACCOUNT )
+    if (gnc_ledger_display_type(gsr->ledger) == LD_SINGLE)
     {
-        gsr->summarybar = NULL;
-        return NULL;
+        if (!xaccAccountIsPriced(gnc_ledger_display_leader(gsr->ledger)))
+        {
+            gsr->balance_label    = add_summary_label (summarybar, TRUE, _("Present:"), NULL);
+            gsr->future_label     = add_summary_label (summarybar, TRUE, _("Future:"), NULL);
+            gsr->cleared_label    = add_summary_label (summarybar, TRUE, _("Cleared:"), NULL);
+            gsr->reconciled_label = add_summary_label (summarybar, TRUE, _("Reconciled:"), NULL);
+            gsr->projectedminimum_label  = add_summary_label (summarybar, TRUE, _("Projected Minimum:"), NULL);
+        }
+        else
+        {
+            gsr->shares_label     = add_summary_label (summarybar, TRUE, _("Shares:"), NULL);
+            gsr->value_label      = add_summary_label (summarybar, TRUE, _("Current Value:"), NULL);
+        }
     }
 
-    summarybar = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
-    gtk_box_set_homogeneous (GTK_BOX (summarybar), FALSE);
+    gsr->filter_label = add_summary_label (summarybar, FALSE, "", NULL);
+    gsr->sort_arrow = gtk_image_new_from_icon_name ("image-missing", GTK_ICON_SIZE_SMALL_TOOLBAR);
+    gsr->sort_label = add_summary_label (summarybar, FALSE, _("Sort By: "), gsr->sort_arrow);
 
-    if (!xaccAccountIsPriced(gnc_ledger_display_leader(gsr->ledger)))
-    {
-        gsr->balance_label    = add_summary_label (summarybar, _("Present:"));
-        gsr->future_label     = add_summary_label (summarybar, _("Future:"));
-        gsr->cleared_label    = add_summary_label (summarybar, _("Cleared:"));
-        gsr->reconciled_label = add_summary_label (summarybar, _("Reconciled:"));
-        gsr->projectedminimum_label  = add_summary_label (summarybar, _("Projected Minimum:"));
-    }
-    else
-    {
-        gsr->shares_label     = add_summary_label (summarybar, _("Shares:"));
-        gsr->value_label      = add_summary_label (summarybar, _("Current Value:"));
-    }
+    gnc_widget_set_style_context (GTK_WIDGET(gsr->filter_label), "gnc-class-highlight");
+    gnc_widget_set_style_context (GTK_WIDGET(gsr->sort_arrow), "gnc-class-highlight");
 
     gsr->summarybar = summarybar;
 
